/*
 *  Template MIB group implementation - 802dot11smt.c
 *
 */
#include <net-snmp/net-snmp-config.h>
#include <net-snmp/net-snmp-includes.h>
#include <net-snmp/agent/net-snmp-agent-includes.h>
#include <net-snmp/agent/auto_nlist.h>
#include "util_funcs.h"   
#include "mibHiMIPv6.h"
#include <sys/stat.h>  

#define PROC_MAX_LINE_SIZE 512
#define StatisticsTable_FILE "/proc/net/hmip6_stat"
#define VisitorListTable_FILE "/proc/net/mip6_MIB_VList"



#define INDEXofIF 0

//Debug Macro for MUST REPORT ERROR
#define ERR(x)  
//Debug Macro for each group
#define IR(x)
#define AR(x)		
#define QQ(x) 
#define CAC(x)
#define IFBW(x) 
#define PL(x)	
#define PP(x)

#define StatisticsTable_REFRESH_INTERVAL 	1
#define VisitorListTable_REFRESH_INTERVAL 	1



Mib_StatisticsTable *current_StatisticsTable=NULL; //used to initialize the table
Mib_StatisticsTable *head_StatisticsTable=NULL; //the root of the initialized data table
Mib_StatisticsTable *cur_ptr_StatisticsTable=NULL; //the ptr used to point what position(or candidate) should return 


Mib_VisitorListTable *current_VisitorListTable=NULL; //used to initialize the table
Mib_VisitorListTable *head_VisitorListTable=NULL; //the root of the initialized data table
Mib_VisitorListTable *cur_ptr_VisitorListTable=NULL; //the ptr used to point what position(or candidate) should return 

/*used to return value*/
static unsigned long long_ret;
static struct counter64 l64_ret;
static char str_ret[100];
union
{
  u_int64_t i64;
  struct
  {
  	  unsigned long low;
	  unsigned long high;
  }i64s;
}un;

void init_mibHiMIPv6(void)
{

/* define the structure we're going to ask the agent to register our
   information at */
  struct variable13 mibHiMIPv6_variables[] = {
   
    /* ----  Statistics Table ----    */
   
    { MIPv6_BU_rcvd,		ASN_INTEGER, RONLY, var_HiMIPv6_Statistics_entry, 3, {1,1,1}},
    { MIPv6_BA_rcvd,		ASN_INTEGER, RONLY, var_HiMIPv6_Statistics_entry, 3, {1,1,2}},
    { MIPv6_BR_rcvd,		ASN_INTEGER, RONLY, var_HiMIPv6_Statistics_entry, 3, {1,1,3}},
    { MIPv6_BU_sent,		ASN_INTEGER, RONLY, var_HiMIPv6_Statistics_entry, 3, {1,1,4}},
    { MIPv6_BA_sent,		ASN_INTEGER, RONLY, var_HiMIPv6_Statistics_entry, 3, {1,1,5}},
    { MIPv6_BR_sent,		ASN_INTEGER, RONLY, var_HiMIPv6_Statistics_entry, 3, {1,1,6}},
    { MIPv6_RA_rcvd,		ASN_INTEGER, RONLY, var_HiMIPv6_Statistics_entry, 3, {1,1,7}},
    { HiMIPv6_HiRA_rcvd,		ASN_INTEGER, RONLY, var_HiMIPv6_Statistics_entry, 3, {1,1,8}},     
    { HiMIPv6_HiBU_rcvd,		ASN_INTEGER, RONLY, var_HiMIPv6_Statistics_entry, 3, {1,1,9}},
    { HiMIPv6_HiBI_rcvd,		ASN_INTEGER, RONLY, var_HiMIPv6_Statistics_entry, 3, {1,1,10}},
    { HiMIPv6_HiBU_sent,		ASN_INTEGER, RONLY, var_HiMIPv6_Statistics_entry, 3, {1,1,11}},
    { HiMIPv6_HiBI_sent,		ASN_INTEGER, RONLY, var_HiMIPv6_Statistics_entry, 3, {1,1,12}},
    { HiMIPv6_HiBA_rcvd,		ASN_INTEGER, RONLY, var_HiMIPv6_Statistics_entry, 3, {1,1,13}},
    { HiMIPv6_HiBA_sent,		ASN_INTEGER, RONLY, var_HiMIPv6_Statistics_entry, 3, {1,1,14}},
    { HiMIPv6_HiBA_passed,		ASN_INTEGER, RONLY, var_HiMIPv6_Statistics_entry, 3, {1,1,15}},
    { HiMIPv6_HiBU_passed,		ASN_INTEGER, RONLY, var_HiMIPv6_Statistics_entry, 3, {1,1,16}},
    { HiMIPv6_HiBU_blocked,		ASN_INTEGER, RONLY, var_HiMIPv6_Statistics_entry, 3, {1,1,17}},
   
    /* ---- Visitor List Table ---- */
    
    { HiMIPv6_VList_Index	,	ASN_INTEGER, 	RONLY,var_HiMIPv6_VisitorList_entry,3,{2,1,1}},
    { HiMIPv6_VList_HomeAddress,	ASN_OCTET_STR, 	RONLY,var_HiMIPv6_VisitorList_entry,3,{2,1,2}},
    { HiMIPv6_VList_DstAddress,	ASN_OCTET_STR, 	RONLY,var_HiMIPv6_VisitorList_entry,3,{2,1,3}},
    { HiMIPv6_VList_COA,		ASN_OCTET_STR, 	RONLY,var_HiMIPv6_VisitorList_entry,3,{2,1,4}}, 
    { HiMIPv6_VList_ExpireTimer,	ASN_INTEGER, 	RONLY,var_HiMIPv6_VisitorList_entry,3,{2,1,5}},
    { HiMIPv6_VList_State,		ASN_INTEGER, 	RONLY,var_HiMIPv6_VisitorList_entry,3,{2,1,6}}
    
    
    
    
  };
  
  /* Define the OID pointer to the top of the mib tree that we're
   registering underneath */
  oid mibHiMIPv6_variables_oid[] = { 1,3,6,1,4,1 ,2025};
  
  #define MIBHIMIPV6_OID_LEN 10

 /* register ourselves with the agent to handle our mib tree */
  REGISTER_MIB("mibHiMIPv6", mibHiMIPv6_variables, variable13, mibHiMIPv6_variables_oid);  
  printf("HiMIPv6 MIB Initilized!\n");

}

void free_StatisticsTable(void)
{
	Mib_StatisticsTable *temp;
	while((temp=head_StatisticsTable) != NULL)
	{
		head_StatisticsTable = head_StatisticsTable->next;
		free(temp);
	}
}

void free_VisitorListTable(void)
{
	Mib_VisitorListTable *temp;
	while((temp=head_VisitorListTable) != NULL)
	{
		head_VisitorListTable = head_VisitorListTable->next;
		free(temp);
	}
}

/* Table initialization functions*/
int init_StatisticsTable(void)
{
	static unsigned long StatisticsTable_last_update=0;

	struct stat buf; 
	FILE *fp;
	char str[PROC_MAX_LINE_SIZE];
	char *c;
	char *sep = ","; //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! 
	char *tok, *brkb;
	int table_count=0;
  
	
	if ( stat(StatisticsTable_FILE,&buf) == -1)
		return -1;
	
	if ( buf.st_mtime == StatisticsTable_last_update)
		return 0;
	
	StatisticsTable_last_update = buf.st_mtime;

	
	fp = fopen(StatisticsTable_FILE,"r");
	
	if (fp == NULL)
	{
		printf("the file StatisticsTable_FILE doesn't exist.\n");
		return -1; 	//Fatal Error Occurs.
	}		
	
	free_StatisticsTable();	//Clean out old data.
	
	while ( (c = fgets(str,PROC_MAX_LINE_SIZE,fp)) != NULL)	
	{
		current_StatisticsTable =  (Mib_StatisticsTable *)calloc(1,sizeof(Mib_StatisticsTable));
			
		tok = (char *)strtok_r(str, sep, &brkb);

			
		 //!!!!!!!!!!!!!!!!!!!!!need to check the order
		
		current_StatisticsTable->BU_rcvd = atoi(tok);	tok = (char *)strtok_r(NULL, sep, &brkb);
		current_StatisticsTable->BA_rcvd = atoi(tok);	tok = (char *)strtok_r(NULL, sep, &brkb);
		current_StatisticsTable->BR_rcvd = atoi(tok);	tok = (char *)strtok_r(NULL, sep, &brkb);
		current_StatisticsTable->BU_sent = atoi(tok);	tok = (char *)strtok_r(NULL, sep, &brkb);
		current_StatisticsTable->BA_sent = atoi(tok);	tok = (char *)strtok_r(NULL, sep, &brkb);
		current_StatisticsTable->BR_sent = atoi(tok);	tok = (char *)strtok_r(NULL, sep, &brkb);
		current_StatisticsTable->RA_rcvd = atoi(tok);	tok = (char *)strtok_r(NULL, sep, &brkb);
		current_StatisticsTable->HiRA_rcvd = atoi(tok);	tok = (char *)strtok_r(NULL, sep, &brkb);
		
		
		
		current_StatisticsTable->HiBU_rcvd = atoi(tok);	tok = (char *)strtok_r(NULL, sep, &brkb);
		current_StatisticsTable->HiBI_rcvd = atoi(tok);		tok = (char *)strtok_r(NULL, sep, &brkb);
		current_StatisticsTable->HiBU_sent = atoi(tok);	tok = (char *)strtok_r(NULL, sep, &brkb);
		current_StatisticsTable->HiBI_sent = atoi(tok);		tok = (char *)strtok_r(NULL, sep, &brkb);
		
		current_StatisticsTable->HiBA_rcvd = atoi(tok);	tok = (char *)strtok_r(NULL, sep, &brkb);
		current_StatisticsTable->HiBA_sent = atoi(tok);	tok = (char *)strtok_r(NULL, sep, &brkb);
		current_StatisticsTable->HiBA_passed = atoi(tok);	tok = (char *)strtok_r(NULL, sep, &brkb);
		current_StatisticsTable->HiBU_passed = atoi(tok);	tok = (char *)strtok_r(NULL, sep, &brkb);
		current_StatisticsTable->HiBU_blocked = atoi(tok);
					
		current_StatisticsTable->next = head_StatisticsTable;
		head_StatisticsTable = current_StatisticsTable;
		table_count++;
	}
	
	//printf("Get %d  SMT_SC. \n",policy_count);
	
	fclose(fp);
 
  		
  	return 0;
	
	
}


int init_VisitorListTable(void)
{
	static unsigned long VisitorListTable_last_update=0;

	struct stat buf; 
	FILE *fp;
	char str[PROC_MAX_LINE_SIZE];
	char *c;
	char *sep = ",\n"; //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! 
	char *tok, *brkb;
	int table_count=1;
  
	
	if ( stat(VisitorListTable_FILE,&buf) == -1)
		return -1;
	
	if ( buf.st_mtime == VisitorListTable_last_update)
		return 0;
	
	VisitorListTable_last_update = buf.st_mtime;

	
	fp = fopen(VisitorListTable_FILE,"r");
	
	if (fp == NULL)
	{
		printf("the file VisitorListTable_FILE doesn't exist.\n");
		return -1; 	//Fatal Error Occurs.
	}		
	
	free_VisitorListTable();	//Clean out old data.
	
	while ( (c = fgets(str,PROC_MAX_LINE_SIZE,fp)) != NULL)	
	{
		current_VisitorListTable =  (Mib_VisitorListTable *)calloc(1,sizeof(Mib_VisitorListTable));
			
		current_VisitorListTable->VisitorListIndex = table_count;
		
		tok = (char *)strtok_r(str, sep, &brkb);
		
		//!!!!!!!!!!!!!!!!!!!!!need to check the order
		strcpy(current_VisitorListTable->HomeAddress, tok);  		tok = (char *)strtok_r(NULL, sep, &brkb);
		strcpy(current_VisitorListTable->DestinationAddress, tok);  	tok = (char *)strtok_r(NULL, sep, &brkb);
		strcpy(current_VisitorListTable->CareOfAddress, tok);  		tok = (char *)strtok_r(NULL, sep, &brkb);
		current_VisitorListTable->ExpireTimer = atoi(tok);		tok = (char *)strtok_r(NULL, sep, &brkb);
		current_VisitorListTable->State = atoi(tok);
			
				
		current_VisitorListTable->next = head_VisitorListTable;
		head_VisitorListTable = current_VisitorListTable;
		table_count++;
	}
	

	
	fclose(fp);
 
  		
  	return 0;
	
	
}
	
	
	
u_char	*
var_HiMIPv6_Statistics_entry(struct variable *vp,
	  oid *name,
	  size_t *length,
	  int exact,
	  size_t *var_len,
	  WriteMethod **write_method) 
{
	oid newname[MAX_OID_LEN], lowest[MAX_OID_LEN], *op; //MAX_OID_LEN = 128 
	int i,found,prio; //integer used to check whether init succeed
	u_char *cp;
	Mib_StatisticsTable *now_ptr;
	
	
	
	i = init_StatisticsTable();
	
	if(i == (-1))
	{
		PP(printf("Not get any StatisticsTable!\n");)
		return NULL;
	}
	
	found = 0;
	lowest[0]=9999;
	now_ptr = head_StatisticsTable;
	if (now_ptr == NULL)
	{
		PP(printf("head_StatisticsTable is NULL\n");)
		return NULL;
	}	
	memcpy((char *)newname,(char *)vp->name,(int)vp->namelen*sizeof(oid));
	
	while(1)
	{
		if(now_ptr == NULL)
			break;
		
		op = newname+MIBHIMIPV6_OID_LEN;
		//*op++ = INDEXofIF;
		
		/*
		*op++ = now_ptr->irpolicyIfIndex[strlen(now_ptr->irpolicyIfIndex)-1]-48;
		*op++ = (u_char)now_ptr->irpolicyIfDir;
		*op++ = now_ptr->irpolicyClassID;
		*/
		
		if(exact)
		{
		  if(snmp_oid_compare(newname,MIBHIMIPV6_OID_LEN,name,*length) == 0)
		  {
		    memcpy((char *)lowest,(char *)newname,(int)(MIBHIMIPV6_OID_LEN)*sizeof(oid));
		    cur_ptr_StatisticsTable = now_ptr;//Current Best Matched Table

		    found++;
		    break;
		  }
		  now_ptr=now_ptr->next;
		}
		else
		{
		  if(snmp_oid_compare(newname,MIBHIMIPV6_OID_LEN,name,*length) >0 && snmp_oid_compare(newname,MIBHIMIPV6_OID_LEN,lowest,MIBHIMIPV6_OID_LEN) <0)
		  {
		    memcpy((char *)lowest,(char *)newname,(MIBHIMIPV6_OID_LEN)*sizeof(oid));
		    cur_ptr_StatisticsTable = now_ptr;
		    found++;
		    PP(if(cur_ptr_StatisticsTable == NULL))
		      PP(printf("lowest is null\n");)
		  }
		  now_ptr=now_ptr->next; 		  
		}
	}	//while end
	
	if(found == 0) return(NULL);
	memcpy((char *)name,(char *)lowest,(vp->namelen)*sizeof(oid));
	*length = vp->namelen ;
	*write_method = 0;
	
	
	switch(vp->magic)
	{
		case MIPv6_BU_rcvd:
			long_ret = (unsigned long)cur_ptr_StatisticsTable->BU_rcvd;
           			*var_len = sizeof(long_ret);
            			return (u_char *)&long_ret;      
			
		case MIPv6_BA_rcvd:
			long_ret = (unsigned long)cur_ptr_StatisticsTable->BA_rcvd;
           			*var_len = sizeof(long_ret);
            			return (u_char *)&long_ret;       
             
		case MIPv6_BR_rcvd:
			long_ret = (unsigned long)cur_ptr_StatisticsTable->BR_rcvd;
           			*var_len = sizeof(long_ret);
            			return (u_char *)&long_ret;      
            
            
        		case MIPv6_BU_sent:
           			long_ret = (unsigned long)cur_ptr_StatisticsTable->BU_sent;
           			*var_len = sizeof(long_ret);
            			return (u_char *)&long_ret;          
             
     		case MIPv6_BA_sent:
			long_ret = (unsigned long)cur_ptr_StatisticsTable->BA_sent;
           			*var_len = sizeof(long_ret);
            			return (u_char *)&long_ret;      
             
		case MIPv6_BR_sent:
			long_ret = (unsigned long)cur_ptr_StatisticsTable->BR_sent;
           			*var_len = sizeof(long_ret);
            			return (u_char *)&long_ret;      
		
		case MIPv6_RA_rcvd:
			long_ret = (unsigned long)cur_ptr_StatisticsTable->RA_rcvd;
           			*var_len = sizeof(long_ret);
            			return (u_char *)&long_ret;      
            
		case HiMIPv6_HiRA_rcvd:
			long_ret = (unsigned long)cur_ptr_StatisticsTable->HiRA_rcvd;
           			*var_len = sizeof(long_ret);
            			return (u_char *)&long_ret;      
		
		case HiMIPv6_HiBU_rcvd:
			long_ret = (unsigned long)cur_ptr_StatisticsTable->HiBU_rcvd;
           			*var_len = sizeof(long_ret);
            			return (u_char *)&long_ret;      
			
		case HiMIPv6_HiBI_rcvd:
			long_ret = (unsigned long)cur_ptr_StatisticsTable->HiBI_rcvd;
           			*var_len = sizeof(long_ret);
            			return (u_char *)&long_ret;    
		
		case HiMIPv6_HiBU_sent:
			long_ret = (unsigned long)cur_ptr_StatisticsTable->HiBU_sent;
           			*var_len = sizeof(long_ret);
            			return (u_char *)&long_ret;      
            			
		case HiMIPv6_HiBI_sent:
			long_ret = (unsigned long)cur_ptr_StatisticsTable->HiBI_sent;
           			*var_len = sizeof(long_ret);
            			return (u_char *)&long_ret;      
		
		case HiMIPv6_HiBA_rcvd:
			long_ret = (unsigned long)cur_ptr_StatisticsTable->HiBA_rcvd;
           			*var_len = sizeof(long_ret);
            			return (u_char *)&long_ret;    
		
		case HiMIPv6_HiBA_sent:
			long_ret = (unsigned long)cur_ptr_StatisticsTable->HiBA_sent;
           			*var_len = sizeof(long_ret);
            			return (u_char *)&long_ret;      
		
            		case HiMIPv6_HiBA_passed:
			long_ret = (unsigned long)cur_ptr_StatisticsTable->HiBA_passed;
           			*var_len = sizeof(long_ret);
            			return (u_char *)&long_ret;    
		
		case HiMIPv6_HiBU_passed:
			long_ret = (unsigned long)cur_ptr_StatisticsTable->HiBU_passed;
           			*var_len = sizeof(long_ret);
            			return (u_char *)&long_ret;    	
            		
            		case HiMIPv6_HiBU_blocked:
			long_ret = (unsigned long)cur_ptr_StatisticsTable->HiBU_blocked;
           			*var_len = sizeof(long_ret);
            			return (u_char *)&long_ret;    	
     			  			
	}
	
	
	return NULL;
}	



u_char	*
var_HiMIPv6_VisitorList_entry(struct variable *vp,
	  oid *name,
	  size_t *length,
	  int exact,
	  size_t *var_len,
	  WriteMethod **write_method) 
{
	oid newname[MAX_OID_LEN], lowest[MAX_OID_LEN], *op; //MAX_OID_LEN = 128 
	int i,found,prio; //integer used to check whether init succeed
	u_char *cp;
	Mib_VisitorListTable *now_ptr;
	
	
	//printf("Entering var_policy_entry!\n");
	
	i = init_VisitorListTable();
	
	if(i == (-1))
	{
		PP(printf("Not get any Visitor List Table!\n");)
		return NULL;
	}
	
	found = 0;
	lowest[0]=9999;
	now_ptr = head_VisitorListTable;
	if (now_ptr == NULL)
	{
		PP(printf("head_VisitorListTable is NULL\n");)
		return NULL;
	}	
	memcpy((char *)newname,(char *)vp->name,(int)vp->namelen*sizeof(oid));
	
	while(1)
	{
		if(now_ptr == NULL)
			break;
		
		op = newname+MIBHIMIPV6_OID_LEN;
		//*op++ = INDEXofIF;
		*op++ = (u_char)now_ptr->VisitorListIndex;
		
		/*
		*op++ = now_ptr->irpolicyIfIndex[strlen(now_ptr->irpolicyIfIndex)-1]-48;
		*op++ = (u_char)now_ptr->irpolicyIfDir;
		*op++ = now_ptr->irpolicyClassID;
		*/
		
		if(exact)
		{
		  if(snmp_oid_compare(newname,MIBHIMIPV6_OID_LEN+1,name,*length) == 0)
		  {
		    memcpy((char *)lowest,(char *)newname,(int)(MIBHIMIPV6_OID_LEN+1)*sizeof(oid));
		    cur_ptr_VisitorListTable = now_ptr;//Current Best Matched Table
		    found++;
		    break;
		  }
		  now_ptr=now_ptr->next;
		}
		else
		{
		  if(snmp_oid_compare(newname,MIBHIMIPV6_OID_LEN+1,name,*length) >0 && snmp_oid_compare(newname,MIBHIMIPV6_OID_LEN+1,lowest,MIBHIMIPV6_OID_LEN+1) <0)
		  {
		    memcpy((char *)lowest,(char *)newname,(MIBHIMIPV6_OID_LEN+1)*sizeof(oid));
		    //PP(printf("next: cur_ptr_table is %s\n",now_ptr->StationID));
		    cur_ptr_VisitorListTable = now_ptr;
		    found++;
		    //PP(if(cur_ptr_AuthenticationAlgorithmsTable == NULL))
		      //PP(printf("lowest is null\n");)
		  }
		  now_ptr=now_ptr->next; 		  
		}
	}	//while end
	
	if(found == 0) return(NULL);
	memcpy((char *)name,(char *)lowest,(vp->namelen+1)*sizeof(oid));
	*length = vp->namelen +1;
	*write_method = 0;
	
	
	switch(vp->magic)
	{
		case HiMIPv6_VList_HomeAddress:
			sprintf(str_ret,"%s\0",cur_ptr_VisitorListTable->HomeAddress);
			*var_len = strlen(str_ret);
			return (u_char *)&str_ret;
		
		case HiMIPv6_VList_DstAddress:
			sprintf(str_ret,"%s\0",cur_ptr_VisitorListTable->DestinationAddress);
			*var_len = strlen(str_ret);
			return (u_char *)&str_ret;
		
		case HiMIPv6_VList_COA:
			sprintf(str_ret,"%s\0",cur_ptr_VisitorListTable->CareOfAddress);
			*var_len = strlen(str_ret);
			return (u_char *)&str_ret;
            
		case HiMIPv6_VList_ExpireTimer:
			long_ret = (unsigned long)cur_ptr_VisitorListTable->ExpireTimer ;
           			*var_len = sizeof(long_ret);
            			return (u_char *)&long_ret;      
		
		case HiMIPv6_VList_State:
			long_ret = (unsigned long)cur_ptr_VisitorListTable->State;
           			*var_len = sizeof(long_ret);
            			return (u_char *)&long_ret;     
            			 	
		
            
			
	}
	
	
	return NULL;
}

